<!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<style>
.r1 {{color: #008080; text-decoration-color: #008080; font-weight: bold}}
.r2 {{color: #800000; text-decoration-color: #800000; font-weight: bold}}
.r3 {{color: #008000; text-decoration-color: #008000}}
.r4 {{color: #008000; text-decoration-color: #008000; font-weight: bold}}
.r5 {{color: #800080; text-decoration-color: #800080}}
.r6 {{color: #ff00ff; text-decoration-color: #ff00ff}}
.r7 {{color: #008080; text-decoration-color: #008080}}
.r8 {{color: #808000; text-decoration-color: #808000}}
.r9 {{color: #800000; text-decoration-color: #800000}}
.r10 {{color: #008080; text-decoration-color: #008080; background-color: #ffffff; font-weight: bold}}
.r11 {{background-color: #ffffff}}
.r12 {{color: #444444; text-decoration-color: #444444; background-color: #ffffff}}
.r13 {{color: #cc3366; text-decoration-color: #cc3366; background-color: #ffffff}}
.r14 {{color: #222222; text-decoration-color: #222222; background-color: #ffffff}}
.r15 {{color: #006699; text-decoration-color: #006699; background-color: #ffffff}}
.r16 {{color: #228822; text-decoration-color: #228822; background-color: #ffffff}}
.r17 {{color: #aa9900; text-decoration-color: #aa9900; background-color: #ffffff}}
.r18 {{color: #6633cc; text-decoration-color: #6633cc; background-color: #ffffff}}
.r19 {{color: #808000; text-decoration-color: #808000; font-weight: bold}}
.r20 {{color: #333333; text-decoration-color: #333333; background-color: #ffffff; font-weight: bold}}
.r21 {{color: #999999; text-decoration-color: #999999; background-color: #ffffff}}
.r22 {{color: #222222; text-decoration-color: #222222}}
.r23 {{font-weight: bold}}
body {{
    color: #000000;
    background-color: #ffffff;
}}
</style>
</head>
<body>
    <pre style="font-family:Menlo,'DejaVu Sans Mono',consolas,'Courier New',monospace"><code style="font-family:inherit">{code}</code></pre>
</body>
</html>